function [] = apply(pInterimFilename)
%APPLY Applies all migrations that are required to upgrade to the latest version.
%   

global artoaWorkspace artoaDataInput artoaConfig;

workspaceSaved = false;
success = true;

%% Check if workspace needs to be saved
if ~isempty(artoaWorkspace) && ~isempty(artoaDataInput) && ~isempty(artoaConfig)
    workspaceSaved = true;
end
%% Save workspace
if workspaceSaved
    disp('Saving workspace...');
    tmpWorkspace = artoaWorkspace;
    tmpDataInput = artoaDataInput;
    tmpConfig = artoaConfig;
    artoaWorkspace = [];
    artoaDataInput = [];
    artoaConfig = [];
    disp('...success!');
end

try
    %% Check if file exists
    if (~isfile(pInterimFilename))
        warning('The given file does not exist! Aborting...');
        restoreWorkspace();
        return
    end

    %% Find all versions available
    [versionsFilePath, ~, ~] = fileparts(mfilename('fullpath'));
    versionsFileNames = dir(fullfile(versionsFilePath, 'versions', 'v*.m'));

    %% Load the interim file
    load(pInterimFilename, 'artoaWorkspace', 'artoaDataInput', 'artoaConfig', '-mat');

    %% Get itm version
    itmVersion = 'unknown';
    if isfield(artoaWorkspace, 'artoaVersion')
        itmVersion = artoaWorkspace.artoaVersion;
    end
    disp(['Starting migration from Version ' itmVersion ' to Version ' versionsFileNames(end).name(2:end - 2)]);

    %% Modify required variables
    for o = 1:size(versionsFileNames)
        if (versionsFileNames(o).isdir)
            warning([versionsFileNames(o).name 'is a directory and will be omitted...']);
            continue;
        end
        currentVersionNumber = versionsFileNames(o).name(2:end - 2);
        if ~strcmp('unknown', itmVersion) & (str2double(currentVersionNumber) <= str2double(itmVersion))
            continue;
        end
        disp(['Applying Version' currentVersionNumber '...']);
        run(fullfile(versionsFileNames(o).folder, versionsFileNames(o).name));
        artoaWorkspace.artoaVersion = currentVersionNumber;
        disp('...done!');
    end

    %% Save with added version number

    [path, name, ext] = fileparts(pInterimFilename);
    adjustedFilename = fullfile(path, [name '_' versionsFileNames(end).name(2:end - 2) ext]);
    save( ...
        adjustedFilename, ...
        'artoaWorkspace', 'artoaDataInput', 'artoaConfig', ...
        '-mat' ...
        );
catch Ex
    disp(Ex);
    success = false;
end
    
%% Restore workspace
if workspaceSaved
    restoreWorkspace();
else
    clearvars -global artoaWorkspace artoaDataInput artoaConfig;
end

if success
    disp(['Adjusted interim file has been written to:' newline adjustedFilename]);
end


%% Nested functions

    function restoreWorkspace()
        disp('Restoring workspace...');
        artoaWorkspace = [];
        artoaDataInput = [];
        artoaConfig = [];
        artoaWorkspace = tmpWorkspace;
        artoaDataInput = tmpDataInput;
        artoaConfig = tmpConfig;
        disp('...success!');
    end

end

