function [ messages, folder ] = fetch_messages( p_imapstore, p_foldername, p_searchterm, p_read_only )
%FETCH_UNREAD Fetches all messages in the given folder.
%   Fetches all messages that match the given searchterm from the given
%   folder.
%
%   Parameters:
%   p_imapstore:            The imapstore object of javax.mail library.
%   p_foldername:           The foldername where the messages will be
%                           fetched from.
%   p_searchterm:           The searchterm created with javax.mail library.
%   p_read_only:            If true, the folder will be opened as read
%                           only.
%   
%   Returns:
%   messages:               The messages that match the searchterm.
%   folder:                 The folder object.

%% Initialize return variables
messages = false;

if (~exist('p_read_only', 'var'))
    p_read_only = true;
end

%% Parameter check
parameter_error = false;

if (~ischar(p_foldername))
    parameter_error = true;
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Check connection

if (~p_imapstore.isConnected())
    disp('IMAPStore not connected, trying to reconnect...');
    p_imapstore.connect();
    if (~p_imapstore.isConnected())
        warning('IMAPStore reconnect failed. Please check your configuration!');
        return;
    else
        disp('... success!');
    end
end

%% Get and open folder
folder = p_imapstore.getFolder(p_foldername);
if (~p_read_only)
    folder.open(folder.READ_WRITE);
else
    folder.open(folder.READ_ONLY);
end

%% Fetch messages
if (isa(p_searchterm, 'logical') || strcmp(p_searchterm, ''))
    messages = folder.getMessages();
else
    messages = folder.search(p_searchterm);
end


end

